@@ -14,7 +14,7 @@
 #    You should have received a copy of the GNU General Public License
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-enable_logging = False
+enable_logging = True
 
 if enable_logging:
     fd = open('/tmp/openvpn-applet.log','w+')
@@ -29,7 +29,6 @@ import gtk
 import hildondesktop
 import hildon
 import gobject
-import osso
 from sets import Set
 import fileinput
 import re
@@ -230,36 +229,34 @@ class vpn_control():
     def get_vpn_count(self):
         return len(self.vpn_list)
 
+    def get_active_connections_number(self):
+        active = 0
+        for i,v in enumerate(self.vpn_list):
+            if v.get_status(): active = active + 1
+        return active
 
     
-class openvpn_applet(hildondesktop.HomeItem):
+class openvpn_applet(hildondesktop.StatusMenuItem):
     def __init__(self):
-        hildondesktop.HomeItem.__init__(self)
+        hildondesktop.StatusMenuItem.__init__(self)
        
         self.vc = None
         self.banner = None
-        self.timer = None       
-        
-        self.frame = gtk.Frame()
-        self.frame.set_name("osso-speeddial")
-        #self.frame.set_border_width(0)
-        
-        #screen = self.get_screen()
-        #colormap = screen.get_rgba_colormap()
-        #if colormap == None:
-        #    self.frame.set_label("No Alpha")
-        #    colormap = screen.get_rgb_colormap()
-        #else:
-        #    self.frame.set_label("Alpha")
-        #self.frame.set_colormap(colormap)
-        
-        self.hbox = gtk.HBox()
-        self.alignment = gtk.Alignment(0.5,0.5,1.0,1.0)
-        self.alignment.set_padding(15,15,15,15)
-   
-        self.button = gtk.Button(label="Start")
+        self.timer = None
+        self.timeout = 0
+        #self.connection_text = "no connections"
+        self.button_title_text = "OpenVPN"
+
+        self.button = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
+        #self.button.set_text(self.button_title_text, self.connection_text)
+        self.button.set_style(hildon.BUTTON_STYLE_PICKER)
+        self.button.set_alignment(0.2,0.5,1,1)
+        image = gtk.image_new_from_icon_name("openvpn-applet", gtk.ICON_SIZE_BUTTON)
+        self.button.set_image(image)
+        self.button.set_image_position(gtk.POS_LEFT)
+
+        #self.button = gtk.Button("OpenVPN");
         self.button.connect("clicked", self.button_clicked_event)
-        self.hbox.pack_start(self.button)
         
         textcell = gtk.CellRendererText()
         imagecell = gtk.CellRendererPixbuf()
@@ -269,22 +266,15 @@ class openvpn_applet(hildondesktop.HomeItem):
         self.combo.pack_start(textcell, True)
         self.combo.add_attribute(imagecell, 'stock-id',3)
         self.combo.add_attribute(textcell, 'text', 0)
-
-        self.vseparator = gtk.VSeparator()
-        self.hbox.pack_start(self.vseparator)
-
         self.combo.connect("changed", self.combo_changed_event)
-        self.hbox.pack_start(self.combo)
 
-        #self.set_resize_type(hildondesktop.HOME_ITEM_RESIZE_BOTH)
-        self.connect("settings", self.settings_event)
-        self.frame.add(self.alignment)
-        self.alignment.add(self.hbox)
-        self.add(self.frame)
-
-        self.update_list()
+        self.add(self.button)
         self.show_all()
-    
+        self.update_list()
+        self.update_button()
+        #self.set_settings(True)
+        #self.connect("show-settings", self.open_settings)
+     
     def update_list(self,source=None):
         vpn_list = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING,gobject.TYPE_PYOBJECT,gobject.TYPE_STRING,gobject.TYPE_BOOLEAN)
         self.vc = vpn_control()
@@ -308,7 +298,7 @@ class openvpn_applet(hildondesktop.HomeItem):
         
         if self.vc.get_vpn_count() > 0:
             self.combo.set_active(0)
-        self.frame.check_resize()
+        #self.frame.check_resize()
 
     def set_options_visible(self,available):
         model = self.combo.get_model()
@@ -334,36 +324,79 @@ class openvpn_applet(hildondesktop.HomeItem):
         index = self.combo.get_active()
         if index >= 0:
             if index == self.vc.get_vpn_count():
-                self.button.set_label("Open")
+                #self.button.set_label("Open")
+                pass
             elif self.vc.status(index) == False:
-                self.button.set_label("Start")
+                #self.button.set_label("Start")
+                pass
             else:
-                self.button.set_label("Stop")
+                #self.button.set_label("Stop")
+                pass
     
     def button_clicked_event(self,widget):
-        index = self.combo.get_active()
-        if index >= 0  and index != self.vc.get_vpn_count():
-            if self.vc.status(index) == False:
-                if self.vc.user_required(index) and self.vc.pass_required(index):
+        dlg = SelectConnectionDialog(parent=None,vpn_manager=self.vc)            
+        dlg.connect("response",self.manage_connection)
+        dlg.show_all()
+        self.update_button()
+    
+    def update_button(self):
+        log("update_button")
+        self.timeout = self.timeout - 1
+        active = self.vc.get_active_connections_number()
+        if active == 1:
+            self.button.set_text(self.button_title_text, str(active) + ' active connection')
+        elif active > 1:
+            self.button.set_text(self.button_title_text, str(active) + ' active connections')
+        else:
+            self.button.set_text(self.button_title_text, 'no active connections')
+            
+        if self.timeout > 0:
+            self.timer = gobject.timeout_add(500, self.update_button)
+        else:
+            self.timer = None
+            return False
+
+    def settings_event(self, widget, data=None):
+        widget = gtk.MenuItem("Openvpn setup")
+        widget.connect("activate", self.open_settings)
+        return widget
+        
+    def open_settings(self, widget=None, data=None):
+        self.set_options_visible(False)
+        dlg = OpenvpnAppletSettings(model=self.combo.get_model(), applet=self)
+        dlg.run()
+        dlg.destroy()
+        self.set_options_visible(True)
+        self.combo.set_active(0)
+        self.update_button()
+
+    def manage_connection(self, widget, vpn_number):
+        if vpn_number == gtk.RESPONSE_HELP:
+            self.open_settings()
+            return
+
+        if vpn_number >= 0  and vpn_number < self.vc.get_vpn_count():
+            if self.vc.status(vpn_number) == False:
+                if self.vc.user_required(vpn_number) and self.vc.pass_required(vpn_number):
                     (user,passwd,cont) = self.user_pass_dlg(True)
-                elif self.vc.pass_required(index):
+                elif self.vc.pass_required(vpn_number):
                     (user,passwd,cont) = self.user_pass_dlg(False)
                 else:
                     user   = None
                     passwd = None
                     cont = True
                 if cont:
-                    self.banner = hildon.hildon_banner_show_information(self,None,"Starting Openvpn Connection")
-                    self.vc.start(index,user,passwd)
+                    self.banner = hildon.hildon_banner_show_information(self, "", "Starting Openvpn Connection")
+                    self.vc.start(vpn_number,user,passwd)
             else:
-                self.banner = hildon.hildon_banner_show_information(self,None,"Stopping Openvpn Connection")
-                self.vc.stop(index)
-            self.timeout = 50
-            self.timer = gobject.timeout_add(200, self.update_button)
-        elif index == self.vc.get_vpn_count():
+                self.banner = hildon.hildon_banner_show_information(self,"", "Stopping Openvpn Connection")
+                self.vc.stop(vpn_number)
+            self.timeout = 10
+            self.timer = gobject.timeout_add(500, self.update_button)
+        elif vpn_number == self.vc.get_vpn_count():
             print "open settings"
             self.open_settings(self)
-    
+
     def user_pass_dlg(self,ask_passwd):
         if ask_passwd:
             dlg = hildon.LoginDialog(gtk.Window())
@@ -384,43 +417,39 @@ class openvpn_applet(hildondesktop.HomeItem):
             user_pass_resp = (user_pass_resp[0],user_pass_resp[1],False)
 
         return user_pass_resp
-    
-    def update_button(self):
-        self.timeout = self.timeout - 1
-        index = self.combo.get_active()
-        if index == self.vc.get_vpn_count():
-            self.timeout = 0
-            self.timer = None
-            return False
-            
-        if self.timeout > 0:
-            if self.vc.status(index) == False:
-                new_label = "Start"
-            else:
-                new_label = "Stop"
-            
-            if self.button.get_label() != new_label:
-                self.update_list()
-                self.combo.set_active(index)
-                self.timeout = 0
-                self.timer = None
-            else:
-                self.timer = gobject.timeout_add(200, self.update_button)
-        else:
-            return False
 
-    def settings_event(self, widget, data=None):
-        widget = gtk.MenuItem("Openvpn setup")
-        widget.connect("activate", self.open_settings)
-        return widget
-        
-    def open_settings(self, widget, data=None):
-        self.set_options_visible(False)
-        dlg = OpenvpnAppletSettings(model=self.combo.get_model(), applet=self)
-        dlg.run()
-        dlg.destroy()
-        self.set_options_visible(True)
-        self.combo.set_active(0)
+class SelectConnectionDialog(gtk.Dialog):
+    def __init__(self,title="Select OpenVPN Connection", parent=None, flags=0, buttons=None,vpn_manager=None):
+        gtk.Dialog.__init__(self,title, parent, flags, buttons)
+        self.vpn_manager = vpn_manager
+        vbox = gtk.VBox()
+        pannable_area = hildon.PannableArea()
+
+        for vpn_number,v in enumerate(vpn_manager.vpn_list):
+            button = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_HORIZONTAL)
+            if v.get_status():
+                button.set_text("Disconnect from",v.get_name())
+            else:
+                button.set_text("Connect to",v.get_name())
+            vbox.pack_start(button)
+            button.connect("clicked", self.button_clicked_event, vpn_number)
+
+        button = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_HORIZONTAL)
+        button.set_text("Manage Connections","")
+        button.connect("clicked", self.button_clicked_event, gtk.RESPONSE_HELP)
+        vbox.pack_start(button)
+
+        pannable_area.add_with_viewport(vbox)
+        self.vbox.pack_start(pannable_area)
+        self.action_area.show_all()
+        self.vbox.show_all()
+        self.set_size_request(800,(vpn_manager.get_vpn_count()+1)*50+35)
+        #self.show_all()
+
+
+    def button_clicked_event(self, widget, vpn_number):
+        self.response(vpn_number)
+        self.destroy()
 
 class OpenvpnAppletSettings(gtk.Dialog):
     def __init__(self,title="Openvpn connections", parent=None, flags=0, buttons=None, model=None, applet=None):
@@ -438,13 +467,13 @@ class OpenvpnAppletSettings(gtk.Dialog):
         self.vpncolumn.pack_start(self.cell, True)
         self.vpncolumn.add_attribute(self.cell, 'text', 0)
 
-        self.new_button = gtk.Button("New")
+        self.new_button = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL,"New")
         self.new_button.connect("clicked", self.open_new_dialog)
-        self.delete_button = gtk.Button("Delete")
+        self.delete_button = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL,"Delete")
         self.delete_button.connect("clicked", self.delete_clicked_event)
         #self.disable_button = gtk.Button("Disable")
         #self.disable_button.connect("clicked", self.disable_clicked_event)
-        self.test_button = gtk.Button("Test")
+        self.test_button = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL,"Test")
         self.test_button.connect("clicked", self.test_clicked_event)
 
 
@@ -575,10 +604,11 @@ class OpenvpnAppletTestConnection(gtk.Dialog):
         
         self.vbox.pack_start(scrolledwindow)
                 
-        save_button = gtk.Button("Save log")
+        save_button = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
+        save_button.set_text("Save Log","")
         save_button.connect("clicked", self.save_button_clicked)
         
-        self.action_area.pack_start(save_button)
+        #self.action_area.pack_start(save_button)
         self.add_button(gtk.STOCK_CLOSE,gtk.RESPONSE_CLOSE)
         
         self.update_text()
@@ -595,7 +625,7 @@ class OpenvpnAppletTestConnection(gtk.Dialog):
         self.timer = gobject.timeout_add(1000, self.update_text)
     
     def save_button_clicked(self, widget, data=None):
-        dlg = hildon.FileChooserDialog(self,gtk.FILE_CHOOSER_ACTION_SAVE)
+        dlg = hildon.FileChooserDialog(self,gtk.FILE_CHOOSER_ACTION_SAVE,  hildon.FileSystemModel())
         response = dlg.run()
         if response == gtk.RESPONSE_OK:
             filename = dlg.get_filename()
@@ -617,44 +647,51 @@ class OpenvpnAppletNewConnection(gtk.Dialog):
 
         self.extra_files = []
         self.check = {}
-        self.file_entry = {}        
+        self.file_entry = {}
+        self.browse_button = {}
 
-        self.add_button("Import Selected",gtk.RESPONSE_OK)
+        self.add_button("Import",gtk.RESPONSE_OK)
         self.add_button("Cancel",gtk.RESPONSE_CANCEL)
-        description_keys = ["Configuration File:", "Key File:", "Cert File:", "Ca File:", "Secret File:", "PKCS12 File:"]
+        description_keys = ["Configuration File", "Key File", "Cert File", "Ca File", "Secret File", "PKCS12 File"]
         description_values = ["conf", "key", "cert", "ca", "secret", "pkcs12"]
         descriptions = {}
         for i,key in enumerate(description_keys):
             descriptions[key] = description_values[i]
         log(descriptions)
         table = gtk.Table(len(descriptions), 2)
+        vbox2 = gtk.VBox()
+        pannable_area = hildon.PannableArea()
         for i,d in enumerate(description_keys):
             log(d)
             hbox = gtk.HBox()
             self.file_entry[descriptions[d]] = gtk.Entry()
-            browse_button = gtk.Button("Browse")
-
-            hbox.pack_start(self.file_entry[descriptions[d]])
-            hbox.pack_start(browse_button)           
+            self.browse_button[descriptions[d]] = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL,d, "None")
+            self.browse_button[descriptions[d]].set_alignment(0.0,0.5,1,1)
+            self.browse_button[descriptions[d]].set_style(hildon.BUTTON_STYLE_PICKER)
+            #hbox.pack_start(self.file_entry[descriptions[d]])
+            hbox.pack_start(self.browse_button[descriptions[d]])           
             self.check[descriptions[d]] = gtk.CheckButton(d)
-            browse_button.connect("clicked", self.browse_button_event, (self.file_entry[descriptions[d]], self.check[descriptions[d]]))   
+            self.browse_button[descriptions[d]].connect("clicked", self.browse_button_event, (self.file_entry[descriptions[d]], self.check[descriptions[d]]))   
 
-            table.attach(self.check[descriptions[d]],0,1,i,i+1)
+            #table.attach(self.check[descriptions[d]],0,1,i,i+1)
             table.attach(hbox,1,2,i,i+1)   
-            self.vbox.pack_start(table)
-            
+            vbox2.pack_start(table)
+
+        pannable_area.add_with_viewport(vbox2)
+        self.vbox.pack_start(pannable_area)
+        self.set_size_request(800,480)
         self.vbox.show_all()
 
     def get_filenames(self):
         files = {}
-        for i,key in enumerate(self.check):
-            if self.check[key].get_active():
-                fname = self.file_entry[key].get_text()
+        for i,key in enumerate(self.browse_button):
+            if self.browse_button[key].get_value() != "None":
+                fname = self.browse_button[key].get_value()
                 perm = '600'
                 for ext in ["conf", "ovpn"]:
                     if re.search("\."+ext+"$",fname):
                         perm = '644'
-                files[self.file_entry[key].get_text()] = perm
+                files[self.browse_button[key].get_value()] = perm
             
             for file in self.extra_files:
                 files[file] = '755'
@@ -664,33 +701,37 @@ class OpenvpnAppletNewConnection(gtk.Dialog):
     def browse_button_event(self, widget, data=None):
         textentry = data[0]
         checkbox = data[1]
-        dlg = hildon.FileChooserDialog(self,gtk.FILE_CHOOSER_ACTION_OPEN)
+        dlg = hildon.FileChooserDialog(self, gtk.FILE_CHOOSER_ACTION_OPEN, hildon.FileSystemModel())
         response = dlg.run()
         if response == gtk.RESPONSE_OK:
+            widget.set_value(dlg.get_filename())
             textentry.set_text(dlg.get_filename())
             checkbox.set_active(True)
             filelinks = FileLinks(dlg.get_filename())
             filelinks.filter(['/home','/media'])
-            print "Referenced files in",dlg.get_filename(),filelinks
+            log("Referenced files in",dlg.get_filename(),filelinks)
             for f in filelinks[dlg.get_filename()]:
-                print "Keyword",f,filelinks.get_matchword(f)
+                log("Keyword",f,filelinks.get_matchword(f))
                 try:
+                    self.browse_button[filelinks.get_matchword(f)].set_value(f)
                     self.file_entry[filelinks.get_matchword(f)].set_text(f)
                     self.check[filelinks.get_matchword(f)].set_active(True)
-                    print "Keyword file:",filelinks.get_matchword(f),f
+                    log("Keyword file:",filelinks.get_matchword(f),f)
                 except KeyError:
                     self.extra_files.append(f)
-                    print "Extra file:",f
+                    log("Extra file:",f)
                 
             if len(filelinks.get_not_found()) > 0:
-                print "Files not found:",filelinks.get_not_found()
+                log("Files not found:",filelinks.get_not_found())
                 mstr = ""
                 for f in filelinks.get_not_found():
-                    mstr = mstr+"\n"+f
+                    if isinstance(f,str):
+                        mstr = mstr+"\n"+f
                 mdlg = gtk.MessageDialog(self,0,gtk.MESSAGE_INFO,gtk.BUTTONS_OK,"Files not found:\n"+mstr)
                 response = mdlg.run()
                 mdlg.destroy() 
-            
+        else:
+            widget.set_value("None")    
         dlg.destroy()
         
 
@@ -716,9 +757,9 @@ class FileLinks(UserDict):
         self.references = {}
         if self.type == 'directory':
             for root, dirs, files in os.walk(self.path):
-                print root, files
+                log(root, files)
                 for name in files:
-                    print "name: ", name
+                    log("name: ", name)
                     for type in self.filetypes:
                         if re.search(".*\."+type+"$",name):
                             self.data[os.path.join(root, name)] = self.__scan(os.path.join(root, name), root)
@@ -726,7 +767,7 @@ class FileLinks(UserDict):
             self.data[self.path] = self.__scan(self.path, os.path.dirname(self.path))
         
     def __scan(self,file,root):
-        print "function: __scan", file, root
+        log("function: __scan", file, root)
         fileset = Set()
         for line in fileinput.input(file):
             parts = line.split()
@@ -748,7 +789,7 @@ class FileLinks(UserDict):
                         elif filename !=None and os.path.exists(os.path.join(root, filename)):
                             fileset.add(os.path.join(root, filename))
                             self.matchword[os.path.join(root, filename)] = word
-                        else:
+                        elif filename !=None:
                             self.files_not_found.append(filename)
                             filename = None
                     
@@ -789,24 +830,33 @@ class FileLinks(UserDict):
                         if re.search('^'+wl,file):
                             passed_files.append(file)
                         else:
-                            print "Dropped:", file
+                            log("Dropped:", file)
             self.data[key] = passed_files
                         
     
-def hd_plugin_get_objects():
-	plugin = openvpn_applet()
-	return [plugin]
+hd_plugin_type = openvpn_applet
 
-def log(input):
+def log(*input):
     if enable_logging:
         print input
         print >>fd,input
+        fd.flush()
 
-if __name__ == '__main__':
-    vc = vpn_control()
-    print vc
-    sel = int(raw_input("command: "))-1
-    if vc.status(sel) != 0:
-        vc.start(sel)
-    else:
-        vc.stop(sel)
+# The code below is just for testing purposes.
+# It allows to run the widget as a standalone process.
+if __name__ == "__main__":
+    import gobject
+    gobject.type_register(hd_plugin_type)
+    obj = gobject.new(hd_plugin_type, plugin_id="plugin_id")
+    obj.show_all()
+    gtk.main()
+
+
+#if __name__ == '__main__':
+#    vc = vpn_control()
+#    print vc
+#    sel = int(raw_input("command: "))-1
+#    if vc.status(sel) != 0:
+#        vc.start(sel)
+#    else:
+#        vc.stop(sel)
